function [lambda, alpha, omega, sizeList] = swirlMle(Z)
% function [lambda, alpha, omega, sizeList] = swirlMle(Z)
%  MLE Parameter estimation for sampling with reduced replacement (SWIRL). Implements section 2.2
%  of Jun et al., 2013.
%
% Input
%
% Z: input sequences, assumed to be generated by SWIRL.
%    N*nmax sparse matrix
%    The j-th row (j=1..N) is the j-th sequence Z^(j)=z_1^(j) ... z_{n(j)}^(j) which is n(j) elements long.
%    nmax >= max_{j=1..N} n(j).  it's OK to have zero padding in all rows.
%    The elements z_i^(j) are word type indices, encoded as positive integers 1, 2, 3, ...
%    Note: the variable-length sequences are padded with zero at the end.
%    Note: Z(j,i) corresponds to z_i^{(j)} in paper
%    Note: It's OK to have empty sequences
%
% Output
%
% lambda: Poisson parameter for sequence length
% alpha: discount factor
% omega: initial weights of items, normalized to form a probability distribution
% sizeList: unnormalized omega (the use of this is rare).
%
% Reference
%
% Kwang-Sung Jun, Xiaojin Zhu, Burr Settles, and Timothy Rogers. Learning from Human-Generated 
% Lists. In The 30th International Conference on Machine Learning (ICML), 2013.
%
% Authors
%
% Jerry Zhu, July 2012
% modified by Kwang-Sung Jun, Sep 2012.

[N, nmax]=size(Z); % N: number of sequences; nmax: length of the longest sequence
nvoc=max(max(Z)); % vocabulary size

%--- construct the C matrices
% C{j}, j=1..N, is a nvoc*n(j) matrix for the j-th sequence.
% The (w,i)-th entry of the above matrix is the number of times w appears in z_1, ... z_{i-1}, the prefix of the j-th sequence
% Note: C{j}(w,i) corresponds to c(w,j,i) in paper

C=cell(N, 1);
n=zeros(N,1); % the length of each sequence

for j=1:N,
  n(j) = length(find(Z(j,:)));
  tmpC = sparse(nvoc, n(j));
  for i=2:n(j),
    tmpC(:,i) = tmpC(:,i-1);
    tmpC(Z(j,i-1),i) = tmpC(Z(j,i-1),i)+1;
  end
  C{j} = tmpC;
end

% The MLE Poisson parameter for sequence length is simply the sample mean
lambda = mean(n);

% compute the shorthand a and b
a = 0;
b = sparse(nvoc, 1);
for j=1:N,
  for i=1:n(j),
    a = a - C{j}(Z(j,i),i);
    b(Z(j,i)) = b(Z(j,i)) - 1;
  end
end

% note b is the negative vocabulary count vector
% we find the most frequent word type (MFW) and later constrain omega(MFW)=1, equivalently x(MFW)=0
[negmfwcount MFW] = min(b);

v0 = [0; log(b / negmfwcount)]; % initial point: alpha=1, omega=with replacement MLE (but scaled so that omega(MFW)=1)
Aeq = zeros(1,nvoc+1); Aeq(1,1+MFW)=1; beq = 0; % equality constraint: x(MFW)=0 so that we remove scale invariance in omega
options = optimset('Algorithm', 'active-set', 'GradObj','on', 'DerivativeCheck', 'off', ...
  'TolFun', 1e-12, ...
  'MaxFunEvals',  4000, 'MaxIter', 4000);

% we stack the variables v=[y;x]
  function [f, g] = obj(v)
    y=v(1); x=v(2:end);
    f = a*y + b'*x;
    partialy = a;
    partialx = b;
    for j=1:N,
      for i=1:n(j);
        discounted = exp(C{j}(:,i)*y + x); % discounted omega
        normalizer = sum(discounted);
        f = f + logsumexp(C{j}(:,i)*y + x);
        partialy = partialy + discounted'*C{j}(:,i)/normalizer;
        partialx = partialx + discounted/normalizer;
      end
    end
    g = [partialy; partialx]; % the gradient
  end

lb = -inf(1,nvoc+1);
ub = inf(1,nvoc+1); ub(1) = 0; % variable bound constraint: y<=0 so that alpha<=1
[v,fval,exitflag,output, ~, grad] = fmincon(@obj, v0, [], [], Aeq, beq, lb, ub, [], options);

alpha = exp(v(1));
omega = exp(v(2:end));
sizeList = omega;
omega = omega/sum(omega);

end












